// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tP: PROJECTION;
float4x4 tWVP: WORLDVIEWPROJECTION;

//texture
texture Src <string uiname="Source";>;
sampler SrcSamp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Src);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

// mask texture
texture Mask<string uiname="Background";>;
sampler MaskSamp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Mask);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

//source transformation
float4x4 tTex: TEXTUREMATRIX <string uiname="Source Transform";>;

//mask transformation
float4x4 tMask: TEXTUREMATRIX <string uiname="Background Transform";>;


//pin for the Gamma
float Gain <
    string uiname="Gain";
    float uimin=-1.0;
    float uimax=1.0;
> = 0.0f;

//pin for the Gamma
float Gamma <
    string uiname="Gamma";
    float uimin=0.0;
    float uimax=3.0;
> = 1.0f;

//pin for the Gain
float Threshold <
    string uiname="Threshold";
    float uimin=0.0;
    float uimax=1.0;
> = 0.0f;

//the data structure: "vertexshader to pixelshader"
//used as output data with the VS function
//and as input data with the PS function
struct vs2ps
{
    float4 Pos  : POSITION;
    float2 TexCd : TEXCOORD0;
    float2 MaskCd : TEXCOORD1;
};

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------
vs2ps VS(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0,
    float4 maskCd : TEXCOORD1)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;

    //transform position
    Out.Pos = mul(PosO, tWVP);
    
    //transform texturecoordinates
    Out.TexCd = mul(TexCd, tTex);

    //transform the mask texture cordinates
    Out.MaskCd = mul(TexCd, tMask);
    return Out;
}

// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------

float4 ps_BgSubtraction(vs2ps In): COLOR
{
    //source texture lookup
    float4 col = tex2D(SrcSamp, In.TexCd);
    // the mask texture lookup
    float4 maskcol = tex2D(MaskSamp, In.MaskCd);
    float4 outColor = {0.0f,0.0f,0.0f,1.0f};

    //get luminance
    float LumCol = (0.222f * col.r + 0.707f * col.g + 0.071f * col.b);
    float LumMask = (0.222f * maskcol.r + 0.707f * maskcol.g + 0.071f * maskcol.b);
    float TempCol = 0.0f;

    //Easy Substraction
    TempCol = (LumCol - LumMask);

    //Adjust Output with Gamma Correction and Gain
    if (TempCol < 0) TempCol = pow(TempCol+Gain,1/Gamma);
    else TempCol = pow(TempCol+Gain,Gamma);

    //Threshold
    if (TempCol > Threshold) TempCol = 1.0f;
    else TempCol = 0.0f;

    //Absolute Substraction
    outColor = lerp (0.0f,col,TempCol);
    outColor.a = 1.0f;

    return outColor;
}

float4 ps_SimpleSubtraction(vs2ps In): COLOR
{
    //source texture lookup
    float4 col = tex2D(SrcSamp, In.TexCd);
    // the mask texture lookup
    float4 maskcol = tex2D(MaskSamp, In.MaskCd);
    float4 outColor = {0.0f,0.0f,0.0f,1.0f};

    //get luminance
    float LumCol = (0.222f * col.r + 0.707f * col.g + 0.071f * col.b);
    float LumMask = (0.222f * maskcol.r + 0.707f * maskcol.g + 0.071f * maskcol.b);
    float TempCol = 0.0f;

    //Easy Substraction
    TempCol = (LumCol - LumMask);

    //Adjust Output with Gamma Correction and Gain
    if (TempCol < 0) TempCol = pow((TempCol*-1)+Gain,1/Gamma);
    TempCol = pow((TempCol*1)+Gain,1/Gamma);

    //Threshold
    if (TempCol > Threshold) TempCol = 1.0f;
    else TempCol = 0.0f;

    outColor=TempCol;
    outColor.a = 1.0f;
    
    return outColor;
}


float4 ps_JustAdjust(vs2ps In): COLOR
{

    //source texture lookup
    float4 col = tex2D(SrcSamp, In.TexCd);
    // the mask texture lookup
    float4 maskcol = tex2D(MaskSamp, In.MaskCd);
    float4 outColor = {0.0f,0.0f,0.0f,1.0f};

    //get luminance
    float LumCol = (0.222f * col.r + 0.707f * col.g + 0.071f * col.b);
    float LumMask = (0.222f * maskcol.r + 0.707f * maskcol.g + 0.071f * maskcol.b);
    float TempCol = 0.0f;
    float AdjustCol = 0.0f;

    //Easy Substraction
     TempCol = (LumCol - LumMask);

    //Adjust Output with Gamma Correction and Gain
    if (TempCol < 0) TempCol = pow(TempCol+Gain,1/Gamma);
    else TempCol = pow(TempCol+Gain,Gamma);
    //Adjustment for Bg Livevideo
    AdjustCol = pow((LumCol)+Gain,1/Gamma);
    outColor.rgb  = AdjustCol;

    //Threshold as overlay on livevideo
    if (TempCol > Threshold) outColor.r  += 0.5f;

    return outColor;

}

// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------

technique BgSubtraction
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 ps_BgSubtraction();
    }
}

technique SimpleSubtraction
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 ps_SimpleSubtraction();
    }
}

technique JustAdjust
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 ps_JustAdjust();
    }
}

technique TFixedFunction
{
    pass P0
    {
        //transforms
        WorldTransform[0]   = (tW);
        ViewTransform       = (tV);
        ProjectionTransform = (tP);

        //texturing
        Sampler[0] = (SrcSamp);
        TextureTransform[0] = (tTex);
        TexCoordIndex[0] = 0;
        TextureTransformFlags[0] = COUNT2;
        //Wrap0 = U;  // useful when mesh is round like a sphere
        
        Lighting       = FALSE;

        //shaders
        VertexShader = NULL;
        PixelShader  = NULL;
    }
}
