// perform sobel edge detection algorithm on given texture

// --------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (DX9)
float4x4 tWV: WORLDVIEW;
float4x4 tWVP: WORLDVIEWPROJECTION;
float4x4 tP: PROJECTION;   //projection matrix as set via Renderer (DX9)

//texture
texture Tex1 <string uiname="Texture1";>;

sampler Samp1 = sampler_state
{
    Texture   = (Tex1);
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    AddressU = BORDER;
    AddressV = BORDER;
};

float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;

float Deform = 1;
float Threshold;
float TextureSize = 512;
// -------------------------------------------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// -------------------------------------------------------------------------------------------------------------------------------------

//data that ist returned by the vertexshader
struct VS_OUTPUT
{
    float4 Pos  : POSITION;
    float4 TexC : TEXCOORD0;
};

VS_OUTPUT VS(
    float4 Pos  : POSITION,
    float4 TexC : TEXCOORD)
{
    VS_OUTPUT Out = (VS_OUTPUT)0;

    TexC = mul(TexC, tTex);
    float2 tz = Pos.xy;
    tz.x *= -1;
    TexC -= 0.5;
    TexC.xy += tz * ((sqrt(.5)-length(tz))*Deform);
    Out.TexC = TexC + 0.5;
    Out.Pos  = mul(Pos, tWVP);
    return Out;
}

// --------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------


float4 PSErode(float4 TexC: TEXCOORD0): COLOR0
{
   float4 d;

   float Width = 1 /float (TextureSize/2.0);
   float Height = 1 /float (TextureSize/2.0);


   float2 sampleOffsets[8] =
   {
   -Width,	-Height,
   0.0,		-Height,	 	
   Width,	-Height,	
   -Width,	 0.0,
   Width,	 0.0,
   -Width,	 Height,
   0.0,		 Height,
   Width,	 Height,  	
   };


   int i = 0;
   float2 texCoords;
   float gray = 0;
   bool doErode = 0;
   
   d = tex2D(Samp1, TexC);
   gray = 0.2989*d.r + 0.5870*d.g + 0.1140*d.b;

   if (gray > Threshold)
   {
        for(i = 0; i < 8; i++)
        {
              texCoords = TexC + sampleOffsets[i]; // add sample offsets
              d = tex2D(Samp1, texCoords);
              gray = 0.2989*d.r + 0.5870*d.g + 0.1140*d.b;
              doErode = doErode || (gray < Threshold);
        }
        return !doErode;
   }
   else
      return 0;
}

float4 PSDilate(float4 TexC: TEXCOORD0): COLOR0
{
   float4 d;

   float Width = 1 /float (TextureSize/2.0);
   float Height = 1 /float (TextureSize/2.0);


   float2 sampleOffsets[8] =
   {
   -Width,	-Height,
   0.0,		-Height,
   Width,	-Height,
   -Width,	 0.0,
   Width,	 0.0,
   -Width,	 Height,
   0.0,		 Height,
   Width,	 Height,
   };


   int i = 0;
   float2 texCoords;
   float gray = 0;
   bool doDilate = 0;

   d = tex2D(Samp1, TexC);
   gray = 0.2989*d.r + 0.5870*d.g + 0.1140*d.b;

   if (gray < Threshold)
   {
        for(i = 0; i < 8; i++)
        {
              texCoords = TexC + sampleOffsets[i]; // add sample offsets
              d = tex2D(Samp1, texCoords);
              gray = 0.2989*d.r + 0.5870*d.g + 0.1140*d.b;
              doDilate = doDilate || (gray > Threshold);
        }
        return doDilate;
   }
   else
      return 1;
}

float4 PSMean(float4 TexC: TEXCOORD0): COLOR0
{
   float4 d;

   float Width = 1 /float (TextureSize/2.0);
   float Height = 1 /float (TextureSize/2.0);

   float2 sampleOffsets[9] =
   {
           -Width,	-Height,
           0.0,		-Height,
           Width,	-Height,
           -Width,	 0.0,
           0, 0,
           Width,	 0.0,
           -Width,	 Height,
           0.0,		 Height,
           Width,	 Height,
   };

   float2 texCoords;
   float gray = 0;
   float intensity = 0;

   for(int i=0; i<9; i++)
   {
         texCoords = TexC + sampleOffsets[i]; // add sample offsets
         d = tex2D(Samp1, texCoords);
         gray = 0.2989*d.r + 0.5870*d.g + 0.1140*d.b;
         intensity += gray;
   }

   return (intensity / 9) > Threshold;
}

float4 PSThresholded(float4 TexC: TEXCOORD0): COLOR0
{
   float4 c = tex2D(Samp1, TexC);
   
   float gray = 0.2989*c.r + 0.5870*c.g + 0.1140*c.b;
   return gray > Threshold;
}

float4 PSPassThrough(float4 TexC: TEXCOORD0): COLOR0
{
   return tex2D(Samp1, TexC);
}


// --------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------

technique TPassThrough
{
    pass P0
    {
      TextureTransformFlags[0] = Projected;
      VertexShader = compile vs_1_1 VS();
      PixelShader  = compile ps_2_0 PSPassThrough();
    }
}

technique TThresholded
{
    pass P0
    {
      TextureTransformFlags[0] = Projected;
      VertexShader = compile vs_1_1 VS();
      PixelShader  = compile ps_2_0 PSThresholded();
    }
}

technique TErode
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_b PSErode();
    }
}

technique TDilate
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_b PSDilate();
    }
}

technique TMean
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 PSMean();
    }
}
